"""Module for SyncCommand class."""
from client import Client
from apiclient.discovery import build
from oauth2client import client
import httplib2
import listing
import image
import inapp_product

class SyncCommand(object):

    """The Sync command executes the up-/download of items from play."""

    def __init__(self, package_name, source_directory, upstream,
                 credentials, **options):
        """
        Create new SyncCommand with given params.

        package_name the app package to upload download items to
        credentials = a GoogleCredentials object
        source_directory = the directory to sync from/to
        upstream = uplaod or download items from/to play
        options = additional options
        """
        super(SyncCommand, self).__init__()
        self.package_name = package_name
        self.credentials = credentials
        self.source_directory = source_directory
        self.upstream = upstream
        self.options = options
        self._init_credentials()
        self._initialize_client()

    def _initialize_client(self):
        self.client = Client(self.package_name, self.service)

    def _init_credentials(self):
        http = httplib2.Http()
        http = self.credentials.authorize(http)

        self.service = build('androidpublisher', 'v2', http=http)

    def execute(self):
        """Execute the command."""
        try:
            if self.upstream:
                if self.options['listings'] is True:
                    listing.upload(self.client, self.source_directory)
                    self.client.commit()
                if self.options['images'] is True:
                    image.upload(self.client, self.source_directory)
                    self.client.commit()
                if self.options['inapp'] is True:
                    inapp_product.upload(self.client, self.source_directory)
            else:
                if self.options['listings'] is True:
                    listing.download(self.client, self.source_directory)
                if self.options['images'] is True:
                    image.download(self.client, self.source_directory)
                if self.options['inapp'] is True:
                    inapp_product.download(self.client, self.source_directory)

        except client.AccessTokenRefreshError:
            print(
                'The credentials have been revoked or expired, please re-run'
                'the application to re-authorize')



"""Package for playdeliver modules."""
from pkg_resources import get_distribution, DistributionNotFound
import os.path

try:
    _dist = get_distribution('playdeliver')
    # Normalize case for Windows systems
    dist_loc = os.path.normcase(_dist.location)
    here = os.path.normcase(__file__)
    if not here.startswith(os.path.join(dist_loc, 'playdeliver')):
        # not installed, but there is another version that *is*
        raise DistributionNotFound
except DistributionNotFound:
    __version__ = 'Please install this project with setup.py'
else:
    __version__ = _dist.version

__all__ = ('client', 'file_util', 'image',
           'listing', 'playdeliver', 'sync_command')



#!/usr/bin/env python
"""starting point for the playdeliver tool."""

import os
import sys
from sync_command import SyncCommand
from oauth2client import client


def _load_key(location):
    if location is not None and os.path.isfile(location):
        f = open(location, 'rb')
        key = f.read()
        f.close()
        return key
    else:
        sys.exit("no key file found")


def execute(options):
    """execute the tool with given options."""
    # Load the key in PKCS 12 format that you downloaded from the Google APIs
    # Console when you created your Service account.
    package_name = options['<package>']
    source_directory = options['<output_dir>']

    if options['upload'] is True:
        upstream = True
    else:
        upstream = False

    sub_tasks = {'images': options['--images'], 'listings': options['--listings'], 'inapp': options['--inapp']}
    if sub_tasks == {'images': False, 'listings': False, 'inapp': False}:
        sub_tasks = {'images': True, 'listings': True, 'inapp': True}

    credentials = create_credentials(credentials_file=options['--credentials'],
                                     service_email=options['--service-email'],
                                     service_key=options['--key'])

    command = SyncCommand(
        package_name, source_directory, upstream, credentials, **sub_tasks)
    command.execute()


def create_credentials(credentials_file=None,
                       service_email=None,
                       service_key=None,
                       scope='https://www.googleapis.com/auth/androidpublisher'):
    """
    Create Google credentials object.

    If given credentials_file is None, try to retrieve file path from environment 
    or look up file in homefolder.
    """
    credentials = None
    if service_email is None and service_key is None:
        print(credentials_file)
        if credentials_file is None:
            # try load file from env
            key = 'PLAY_DELIVER_CREDENTIALS'
            if key in os.environ:
                credentials_file = os.environ[key]

        if credentials_file is None:
            # try to find the file in home
            path = os.path.expanduser('~/.playdeliver/credentials.json')
            if os.path.exists(path):
                credentials_file = path

        if credentials_file is not None:
            credentials = client.GoogleCredentials.from_stream(
                credentials_file)
            credentials = credentials.create_scoped(scope)
        else:
            sys.exit("no credentials")
    else:
        credentials = client.SignedJwtAssertionCredentials(
            service_email, _load_key(service_key), scope=scope)
    return credentials



"""This module helps for uploading and downloading listings from/to play."""
import os
import json
from file_util import mkdir_p
from file_util import list_dir_abspath


def upload(client, source_dir):
    """Upload listing files in source_dir. folder herachy."""
    print('')
    print('upload store listings')
    print('---------------------')
    listings_folder = os.path.join(source_dir, 'listings')
    langfolders = filter(os.path.isdir, list_dir_abspath(listings_folder))

    for language_dir in langfolders:
        language = os.path.basename(language_dir)
        with open(os.path.join(language_dir, 'listing.json')) as listings_file:
            listing = json.load(listings_file)
        listing_response = client.update(
            'listings', language=language, body=listing)

        print('  Listing for language %s was updated.' %
              listing_response['language'])


def download(client, target_dir):
    """Download listing files from play and saves them into folder herachy."""
    print('')
    print('download store listings')
    print('---------------------')
    listings = client.list('listings')
    for listing in listings:
        path = os.path.join(target_dir, 'listings', listing['language'])
        mkdir_p(path)
        with open(os.path.join(path, 'listing.json'), 'w') as outfile:
            print("save listing for {0}".format(listing['language']))
            json.dump(
                listing, outfile, sort_keys=True,
                indent=4, separators=(',', ': '))



"""This module helps for uploading and downloading images from/to play."""
import imghdr
import httplib2
import os
from file_util import mkdir_p
from file_util import list_dir_abspath

image_types = ["featureGraphic",
               "icon",
               "phoneScreenshots",
               "promoGraphic",
               "sevenInchScreenshots",
               "tenInchScreenshots",
               "tvBanner",
               "tvScreenshots"]

single_image_types = ['tvBanner',
                      'promoGraphic',
                      'icon',
                      'featureGraphic']


def upload(client, source_dir):
    """
    Upload images to play store.

    The function will iterate through source_dir and upload all matching
    image_types found in folder herachy.
    """
    print('')
    print('upload images')
    print('-------------')
    base_image_folders = [
        os.path.join(source_dir, 'images', x) for x in image_types]

    for type_folder in base_image_folders:
        if os.path.exists(type_folder):
            image_type = os.path.basename(type_folder)
            langfolders = filter(os.path.isdir, list_dir_abspath(type_folder))
            for language_dir in langfolders:
                language = os.path.basename(language_dir)
                delete_and_upload_images(
                    client, image_type, language, type_folder)


def delete_and_upload_images(client, image_type, language, base_dir):
    """
    Delete and upload images with given image_type and language.

    Function will stage delete and stage upload all
    found images in matching folders.
    """
    print('{0} {1}'.format(image_type, language))
    files_in_dir = os.listdir(os.path.join(base_dir, language))
    delete_result = client.deleteall(
        'images', imageType=image_type, language=language)

    deleted = delete_result.get('deleted', list())
    for deleted_files in deleted:
        print('  delete image: {0}'.format(deleted_files['id']))

    for image_file in files_in_dir[:8]:
        image_file_path = os.path.join(base_dir, language, image_file)
        image_response = client.upload(
            'images',
            imageType=image_type,
            language=language,
            media_body=image_file_path)
        print("  upload image {0} new id {1}".format(image_file, image_response['image']['id']))


def download(client, target_dir):
    """Download images from play store into folder herachy."""
    print('download image previews')
    print(
        "Warning! Downloaded images are only previews!"
        "They may be to small for upload.")
    tree = {}
    listings = client.list('listings')
    languages = map(lambda listing: listing['language'], listings)

    parameters = [{'imageType': image_type, 'language': language}
                  for image_type in image_types for language in languages]
    tree = {image_type: {language: list()
                         for language in languages}
            for image_type in image_types}

    for params in parameters:
        result = client.list('images', **params)
        image_type = params['imageType']
        language = params['language']
        tree[image_type][language] = map(
            lambda r: r['url'], result)

    for image_type, language_map in tree.items():
        for language, files in language_map.items():
            if len(files) > 0:
                mkdir_p(
                    os.path.join(target_dir, 'images', image_type, language))
            if image_type in single_image_types:
                if len(files) > 0:
                    image_url = files[0]
                    path = os.path.join(
                        target_dir,
                        'images',
                        image_type,
                        language,
                        image_type)
                    load_and_save_image(image_url, path)
            else:
                for idx, image_url in enumerate(files):
                    path = os.path.join(
                        target_dir,
                        'images',
                        image_type,
                        language,
                        image_type + '_' + str(idx))
                    load_and_save_image(image_url, path)


def load_and_save_image(url, destination):
    """Download image from given url and saves it to destination."""
    from urllib2 import Request, urlopen, URLError, HTTPError
    # create the url and the request
    req = Request(url)

    # Open the url
    try:
        f = urlopen(req)
        print "downloading " + url

        # Open our local file for writing

        local_file = open(destination, "wb")
        # Write to our local file
        local_file.write(f.read())
        local_file.close()

        file_type = imghdr.what(destination)
        local_file = open(destination, "rb")
        data = local_file.read()
        local_file.close()

        final_file = open(destination + '.' + file_type, "wb")
        final_file.write(data)
        final_file.close()
        print('save image preview {0}'.format(destination + '.' + file_type))
        os.remove(destination)

    # handle errors
    except HTTPError, e:
        print "HTTP Error:", e.code, url
    except URLError, e:
        print "URL Error:", e.reason, url



"""module with utility functions for the file system."""

import os
import errno


def list_dir_abspath(path):
    """
    Return a list absolute file paths.

    see mkdir_p os.listdir.
    """
    return map(lambda f: os.path.join(path, f), os.listdir(path))


def mkdir_p(path):
    """Create a new directory with with all missing folders in between."""
    try:
        os.makedirs(path)
    except OSError as exc:  # Python >2.5
        if exc.errno == errno.EEXIST and os.path.isdir(path):
            pass
        else:
            raise



"""Module with Client class to access google api."""


class Client(object):
    """
    Client object which handles google api edits.

    It hold the service user credentials and app package name.
    """

    def __init__(self, package_name, service):
        """
        create new client object.

        package_name = the app package you want to access
        credentials_file = path to credentials json
        email = the service user email
        key = the service user key
        """
        super(Client, self).__init__()
        self.package_name = package_name
        self.service = service
        self.edit_id = None

    def list(self, service_name, **params):
        """
        convinent access method for list.

        service_name describes the endpoint to call
        the `list` function on.

        images.list or apks.list.
        """
        result = self._invoke_call(service_name, 'list', **params)
        if result is not None:
            return result.get(service_name, list())
        return list()

    def list_inappproducts(self):
        """temp function to list inapp products."""

        result = self.service.inappproducts().list(
            packageName=self.package_name).execute()
        if result is not None:
            return result.get('inappproduct', list())
        return list()

    def insert_inappproduct(self, product):
        return self.service.inappproducts().insert(
            packageName=self.package_name, body=product).execute()

    def update_inappproduct(self, product, sku):
        return self.service.inappproducts().update(
            packageName=self.package_name, sku=sku, body=product).execute()

    def update(self, service_name, **params):
        """
        convinent access method for update.

        service_name describes the endpoint to call
        the `update` function on.

        images.update or apks.update.
        """
        return self._invoke_call(service_name, 'update', **params)

    def deleteall(self, service_name, **params):
        """
        convinent access method for deleteall.

        service_name describes the endpoint to call
        the `deleteall` function on.

        images.deleteall or apks.deleteall.
        """
        return self._invoke_call(service_name, 'deleteall', **params)

    def upload(self, service_name, **params):
        """
        convinent access method for upload.

        service_name describes the endpoint to call
        the `upload` function on.

        images.upload or apks.upload.
        """
        return self._invoke_call(service_name, 'upload', **params)

    def commit(self):
        """commit current edits."""
        request = self.edits().commit(**self.build_params()).execute()

        print 'Edit "%s" has been committed' % (request['id'])
        self.edit_id = None

    def _invoke_call(self, service_name, function_name, **params):
        self.ensure_edit_id()
        params = self.build_params(params)
        service_impl = getattr(self.edits(), service_name)
        method = getattr(service_impl(), function_name)

        if method and callable(method):
            return method(**params).execute()
        pass
        return None

    def build_params(self, params={}):
        """
        build a params dictionary with current editId and packageName.

        use optional params parameter
        to merge additional params into resulting dictionary.
        """
        z = params.copy()
        z.update({'editId': self.edit_id, 'packageName': self.package_name})
        return z

    def edits(self):
        """Return current edits object."""
        return self.service.edits()

    def ensure_edit_id(self):
        """create edit id if edit id is None."""
        if self.edit_id is None:
            edit_request = self.edits().insert(
                body={}, packageName=self.package_name)
            result = edit_request.execute()
            self.edit_id = result['id']



"""This module helps for uploading and downloading inappproducts from/to play."""
import os
import json
from file_util import mkdir_p
from file_util import list_dir_abspath

def upload(client, source_dir):
    """Upload inappproducts to play store."""
    print('')
    print('upload inappproducs')
    print('---------------------')

    products_folder = os.path.join(source_dir, 'products')
    product_files = filter(os.path.isfile, list_dir_abspath(products_folder))

    current_product_skus = map(lambda product: product['sku'], client.list_inappproducts())
    print(current_product_skus)
    for product_file in product_files:
        with open(product_file) as product_file:
            product = json.load(product_file)
        #check if the product is new
        sku = product['sku']
        product['packageName'] = client.package_name
        print(sku)
        if sku in current_product_skus:
            print("update product {0}".format(sku))
            client.update_inappproduct(product, sku)
        else:
            print("create product {0}".format(sku))
            client.insert_inappproduct(product)


def download(client, target_dir):
    """Download inappproducts from play store."""
    print('')
    print("download inappproducts")
    print('---------------------')
    products = client.list_inappproducts()

    for product in products:
        path = os.path.join(target_dir, 'products')
        del product['packageName']
        mkdir_p(path)
        with open(os.path.join(path, product['sku'] + '.json'), 'w') as outfile:
            print("save product for {0}".format(product['sku']))
            json.dump(
                product, outfile, sort_keys=True,
                indent=4, separators=(',', ': '))



"""Setup file for playdeliver."""
from __future__ import print_function
from setuptools import setup, find_packages

import io
import codecs
import os
import sys

import playdeliver

here = os.path.abspath(os.path.dirname(__file__))


def _read(*filenames, **kwargs):
    encoding = kwargs.get('encoding', 'utf-8')
    sep = kwargs.get('sep', '\n')
    buf = []
    for filename in filenames:
        with io.open(filename, encoding=encoding) as f:
            buf.append(f.read())
    return sep.join(buf)

long_description = _read('README.rst', 'CHANGES.rst')

setup(
    name='playdeliver',
    version='1.1.0',
    url='https://github.com/wooga/play-deliver',
    download_url='https://github.com/wooga/play-deliver/archive/1.1.0.tar.gz',
    license='MIT LICENSE',
    author='Manfred Endres',
    install_requires=['docopt==0.6.1',
                      'google-api-python-client==1.4.0',
                      'pycrypto',
                      'pyopenssl'],

    author_email='manfred.endres@wooga.net',
    description='google play store delivering tool',
    long_description=long_description,
    packages=['playdeliver'],
    include_package_data=True,
    platforms='any',
    classifiers=[
        'Programming Language :: Python :: 2 :: Only',
        'Development Status :: 5 - Production/Stable',
        'Natural Language :: English',
        'Environment :: MacOS X',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: MacOS :: MacOS X',
        'Topic :: Multimedia :: Graphics',
        'Topic :: System :: Archiving :: Packaging',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    scripts=['bin/playdeliver']
)
